outputs: {
  config,
  lib,
  pkgs,
  ...
}:
with lib; let
  cfg = config.services.nm-reactor;
in {
  options.services.nm-reactor = {
    enable = mkEnableOption "Enables the nm-reactor daemon";

    package = mkOption {
      type = types.package;
      default = outputs.defaultPackage.${pkgs.system};
      description = mdDoc "The `nm-reactor` package to use.";
    };

    configLines = mkOption {
      type = types.lines;
      description = mdDoc "The contents of the configuration file";
    };
  };

  config = mkIf cfg.enable {
    xdg.configFile."nm-reactor/config.yaml" = {
      text = cfg.configLines;
    };

    home.packages = [cfg.package];

    systemd.user.services.nm-reactor = {
      Unit = {
        Description = ''
          A NetworkManager reactor. If-this-then-that for NetworkManager.
        '';
      };

      Service = {
        ExecStart = "${cfg.package}/bin/nm-reactor";
        Environment = "RUST_LOG=info";
      };

      Install.WantedBy = ["default.target"];
    };
  };
}
