use std::{sync::Arc, time::Duration};

use anyhow::Result;
use dbus::{
    nonblock::{Proxy, SyncConnection},
    Path,
};

use nm_reactor_dbus_codegen::network_manager_connection_active::OrgFreedesktopNetworkManagerConnectionActive;

pub struct ActiveConnectionWrapper<'a> {
    proxy: Box<dyn OrgFreedesktopNetworkManagerConnectionActive + Send + Sync + 'a>,
    path: Path<'a>,
}

impl<'a> ActiveConnectionWrapper<'a> {
    pub async fn from_path<P>(conn: Arc<SyncConnection>, path: P) -> ActiveConnectionWrapper<'a>
    where
        P: Into<dbus::Path<'a>>,
    {
        let path = path.into();

        let proxy: Proxy<'a, Arc<SyncConnection>> = Proxy::new(
            "org.freedesktop.NetworkManager",
            path.clone(),
            Duration::from_millis(5000),
            conn,
        );

        ActiveConnectionWrapper {
            proxy: Box::new(proxy),
            path,
        }
    }

    pub fn get_path(&self) -> Path<'a> {
        self.path.clone()
    }

    pub async fn get_uuid(&self) -> Result<String> {
        let uuid = self.proxy.uuid().await?;

        Ok(uuid)
    }

    pub async fn get_device_paths(&self) -> Result<Vec<Path<'static>>> {
        let device_paths = self.proxy.devices().await?;

        Ok(device_paths)
    }
}
