use std::{sync::Arc, time::Duration};

use anyhow::Result;
use dbus::{
    nonblock::{Proxy, SyncConnection},
    Path,
};

use nm_reactor_dbus_codegen::network_manager_settings::OrgFreedesktopNetworkManagerSettings;

use super::connection::ConnectionWrapper;

pub struct ManagerSettingsWrapper<'a> {
    proxy: Box<dyn OrgFreedesktopNetworkManagerSettings + Send + Sync + 'a>,
}

impl<'a> ManagerSettingsWrapper<'a> {
    pub async fn from_connection(conn: &Arc<SyncConnection>) -> ManagerSettingsWrapper<'a> {
        let proxy: Proxy<'a, Arc<SyncConnection>> = Proxy::new(
            "org.freedesktop.NetworkManager",
            "/org/freedesktop/NetworkManager/Settings",
            Duration::from_millis(5000),
            conn.clone(),
        );

        ManagerSettingsWrapper {
            proxy: Box::new(proxy),
        }
    }

    pub async fn get_connection_from_uuid(
        &self,
        uuid: &str,
    ) -> Result<Option<ConnectionWrapper<'static>>> {
        let path = self.proxy.get_connection_by_uuid(uuid).await?;

        if path == Path::from("/") {
            return Ok(None);
        }

        let connection = ConnectionWrapper::from_path(path).await;

        Ok(Some(connection))
    }
}
